#include "StdAfx.h"
#include "Files.h"

#include <algorithm>

/**
 * @brief Get the dos file time of a file.
 * @param FileName fully qualified path.
 */
int FileAge(LPCTSTR FileName)
{
	HANDLE Handle;
	WIN32_FIND_DATA FindData;
	FILETIME LocalFileTime;

	WORD wHigh;
	WORD wLow;

	Handle = FindFirstFile(FileName, &FindData);
	if (Handle != INVALID_HANDLE_VALUE)
	{
		FindClose(Handle);
		if ((FindData.dwFileAttributes & FILE_ATTRIBUTE_DIRECTORY) == 0)
		{
			FileTimeToLocalFileTime(&FindData.ftLastWriteTime, &LocalFileTime);
			if (FileTimeToDosDateTime(&LocalFileTime, &wHigh, &wLow) != 0)
			{
				return (int)MAKELONG(wLow, wHigh);
			}
		}
	}
  return -1;
}

bool FileExists(LPCTSTR FileName)
{
	HANDLE h;
	WIN32_FIND_DATA FindData;

	h = FindFirstFile(FileName, &FindData);
	FindClose(h);
	
	return h != INVALID_HANDLE_VALUE;
}

/**
 * @brief Recursively create a directory.
 * @param pszDirectory full directory path to create.
 * @param lpSA pointer to a SECURITY_ATTRIBUTES structure to be passed to CreateDirectory
 *
 * Credit to Eugen@Fastfertig.com who posted almost exactly this code
 * to CodeGuru. Saved me writing it!
 * link: http://www.codeguru.com/mfc/comments/39715.shtml
 */
bool CreateDirectoryRecursive(LPCTSTR pszDirectory, LPSECURITY_ATTRIBUTES lpSA) 
{
	char szDir[MAX_PATH];
	char *p, *pNext;
	strcpy(szDir, pszDirectory);

	pNext = strchr(szDir, '\\');
	if (pNext)
	{
		pNext++;
		while ( ( p = strchr(pNext, '\\') ) != 0 )
		{
			*p = NULL;
			if (GetFileAttributes(szDir) == -1)
			{
				if (CreateDirectory(szDir, lpSA) == FALSE)
				{
					return FALSE;
				}
			}
			*p = '\\';
			pNext = p+1;
		}
	}
	if (GetFileAttributes (szDir) == -1)
	{
		if (CreateDirectory(szDir, lpSA) == FALSE)
		{
			return false;
		}
	}

	return true;
}

///////////////////////////////////////////////////////////////////////////
// CFile - an MFC CFile replacement. No win api stuff, just pure Cpp.
///////////////////////////////////////////////////////////////////////////

CFile::CFile()
{
	m_file = NULL;
}

CFile::~CFile()
{
	Close();
}

bool CFile::Open(LPCTSTR filename, UINT flags)
{
	CString mode;

	if(flags != 0)
	{
		if(modeWrite & flags)
			mode = _T("w");
		else if(modeReadWrite & flags)
			mode = _T("r+");
		else mode = _T("r");

		if(modeText & flags)
			mode += _T("t");
		else
			mode += _T("b");
	}
	else
		mode = _T("rb");
		

	m_file = _tfopen(filename, mode);
	
	return (m_file != NULL);
}

int CFile::Read(void* lpBuf, UINT nCount)
{
	return (int)fread(lpBuf, 1, nCount, m_file);
}

int CFile::Write(void* lpBuf, UINT nCount)
{
	return (int)fwrite(lpBuf, nCount, 1, m_file);
}

void CFile::Close()
{
	if(m_file != NULL)
	{
		fclose(m_file);
		m_file = NULL;
	}
}

long CFile::GetPosition() const
{
	if(m_file)
	{
		return ftell(m_file);
	}
	else
		return -1;
}

long CFile::GetLength()
{
	if(m_file)
	{
		long curPos = GetPosition();
		fseek(m_file, 0, SEEK_END);
		long ret = GetPosition();
		fseek(m_file, curPos, SEEK_SET);
		return ret;
	}
	else
		return -1;
}

int CFile::ShowError(LPCTSTR filename, bool bOpen)
{
	int err = GetLastError();
	
	TCHAR* fstr;
	
	switch(err)
	{
	case ERROR_ACCESS_DENIED:
	case ERROR_NOT_DOS_DISK:
	case ERROR_WRITE_PROTECT:
		if (bOpen )
			fstr = CFILE_LoadAccessDenied;
		else
			fstr = CFILE_SaveAccessDenied;
		break;
		
	case ERROR_DISK_FULL:
	case ERROR_HANDLE_DISK_FULL:
		if (bOpen)
			fstr = CFILE_CouldNotLoadError;
		else
			fstr = CFILE_SaveDiskFullError;
		break;
		
	case ERROR_SHARING_VIOLATION:
	case ERROR_LOCK_VIOLATION:
		if (bOpen)
			fstr = CFILE_LoadShareViolation;
		else
			fstr = CFILE_SaveShareViolation;
		break;
		
	case ERROR_DEV_NOT_EXIST:
	case ERROR_BAD_NETPATH:
	case ERROR_NETWORK_BUSY:
		if (bOpen)
			fstr = CFILE_NetLoadError;
		else
			fstr = CFILE_NetSaveError;
		break;
		
	default:
		if (bOpen)
			fstr = CFILE_CouldNotLoadError;
		else
			fstr = CFILE_CouldNotSaveError;
	}

	CString str;
	str.Format(fstr, filename);
	if(bOpen)
		return ::MessageBox(NULL, (LPCTSTR)str, _T("Programmers Notepad 2"), MB_OK);
	else
		return ::MessageBox(NULL, (LPCTSTR)str, _T("Programmers Notepad 2"), MB_YESNO);
}

///////////////////////////////////////////////////////////////////////////
// CTextFile - text file operations...
///////////////////////////////////////////////////////////////////////////

// basically taken from MFC's CStdioString...
bool CTextFile::ReadLine(CString& line)
{
	line = _T("");
	const int nMaxSize = 128;
	LPTSTR lpsz = line.GetBuffer(nMaxSize);
	LPTSTR lpszResult;
	int nLen = 0;
	
	for (;;)
	{
		lpszResult = _fgetts(lpsz, nMaxSize+1, m_file);
		line.ReleaseBuffer();

		// handle error/eof case
		if (lpszResult == NULL && !feof(m_file))
		{
			clearerr(m_file);
			// could throw exception here...
		}

		// if string is read completely or EOF
		if (lpszResult == NULL ||
			(nLen = lstrlen(lpsz)) < nMaxSize ||
			lpsz[nLen-1] == '\n')
			break;

		nLen = line.GetLength();
		lpsz = line.GetBuffer(nMaxSize + nLen) + nLen;
	}

	// remove '\n' from end of string if present
	lpsz = line.GetBuffer(0);
	nLen = line.GetLength();
	if (nLen != 0 && lpsz[nLen-1] == '\n')
		line.GetBufferSetLength(nLen-1);

	return lpszResult != NULL;
}

bool CTextFile::WriteLine(LPCTSTR line)
{
	int ret = _fputts(line, m_file);
	return (ret != _TEOF);
}

///////////////////////////////////////////////////////////////////////////
// CFileName - allows lots of useful operations on a filename.
///////////////////////////////////////////////////////////////////////////

CFileName& CFileName::operator = (LPCTSTR filename)
{
	m_FileName = filename;
	return *this;
}

CFileName& CFileName::operator = (const CFileName& filename)
{
	m_FileName = filename.m_FileName;
	return *this;
}

int CFileName::GetLastSlashPos()
{
	int pos = m_FileName.rfind(_T('\\'));
	if(pos == m_FileName.npos)
	{
		// couldn't find '\\', try again for '/'
		pos = m_FileName.rfind(_T('/'));
	}
	return ++pos;
}

int CFileName::GetLastDotPos(tstring* str)
{
	if(!str)
	{
		int pos = m_FileName.rfind(_T('.'));
		if(pos != m_FileName.npos)
			return ++pos;
		else
			return m_FileName.npos;
	}
	else
	{
		int pos = str->rfind(_T('.'));
		if(pos != m_FileName.npos)
			return ++pos;
		else
			return m_FileName.npos;
	}
}

void CFileName::GetPath(tstring& buf)
{
	int pos = GetLastSlashPos();
	if (pos != m_FileName.npos)
	{
		buf = m_FileName.substr(0, pos);
	}
	else
	{
		buf = "";
	}
}

void CFileName::GetFileName(tstring& buf)
{
	buf = GetFileName();
}

tstring CFileName::GetFileName()
{
	int pos = GetLastSlashPos();

	if (pos != m_FileName.npos)
		return m_FileName.substr(pos);
	else
		return m_FileName;
}

///@todo this crashes if the file has no extension!!!
tstring CFileName::GetExtension()
{
	int pos = GetLastDotPos(&m_FileName);
	if(pos != m_FileName.npos)
	{
		return m_FileName.substr(pos-1);
	}
	else
		return tstring("");
}

void CFileName::GetFileName_NoExt(tstring& buf)
{
	tstring work;
	
	int pos = GetLastSlashPos();
	
	if (pos != m_FileName.npos)
	{
		work = m_FileName.substr(pos); 
	}
	else
	{
		work = m_FileName;
	}
	
	pos = GetLastDotPos(&work);
	if(pos != work.npos)
	{
		work = work.substr(0, pos-1);
	}

	buf = work;
}

void CFileName::ChangeExtensionTo(LPCTSTR newext)
{
	tstring	str;
	tstring	buf;
	GetPath(buf);
	str = buf;
	GetFileName_NoExt(buf);
	str += buf;
	str += newext;
	m_FileName = str;
}

void CFileName::ChangePathTo(LPCTSTR newpath)
{
	tstring str;
	GetFileName(str);
	str = newpath + str;
	m_FileName = str;
}

const TCHAR* CFileName::c_str()
{
	return m_FileName.c_str();
}

int CFileName::GetLength()
{
	return m_FileName.length();
}

int CFileName::GetFileAge()
{
	return FileAge(m_FileName.c_str());
}

const tstring& CFileName::ToLower()
{
	transform (m_FileName.begin(), m_FileName.end(),    // source
               m_FileName.begin(),             // destination
               tolower);

	return m_FileName;
}